# -*- coding: utf-8 -*-

import json

import click
from datetime import datetime

from pymongo import UpdateOne
from kombu import Connection
from concurrent.futures import ThreadPoolExecutor
from em_tasks.utils.offer_services import AmzOfferService
from em_tasks.utils.amz_products_analyzer_and_send import AmzProductsAnalyzerAndSend
from em_celery.tasks.spapi_update_item_offers_task import spapi_update_item_offers
from em_tasks.store.models import *
from em_celery.tools.spapi_update_catalog_items_task_send_from_mongo import ensure_target_country_match_queue

from em_celery import (
  logger,
  init_db,
  get_config,
  get_amz_offer_filter_config,
  get_offer_service_config,
  get_product_service,
  get_mongo
)

@click.command("Analyze Amazon products.")
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option("-m", "--marketplace", required=True, type=str, default="us", help="Amazon marketplace.")
def analyze_products(marketplace, broker_url):
  init_db()

  marketplace = marketplace.lower()
  filter_cond = get_amz_offer_filter_config(marketplace)
  offer_service_cfg = get_offer_service_config()
  offer_service = AmzOfferService(
    offer_service_cfg["host"],
    offer_service_cfg["port"],
    offer_service_cfg["user"],
    offer_service_cfg["password"],
    filter_cond,
  )

  logger.info("[OfferFilters] %s", filter_cond)

  product_service = get_product_service()
  mongo_client = get_mongo()
  asins = []

  queue = 'SpapiItemOffersUpdate_{}'.format(marketplace.upper())
  sender = get_offer_sender(marketplace, broker_url, queue)
  mongo_sender = get_mongo_sender(mongo_client, marketplace)

  products_filter = AmzProductsAnalyzerAndSend(marketplace, product_service, offer_service, mongo_client)
  for products_processed in products_filter.run():
      for asin in products_processed:
          if len(asins) < 20:
              asins.append(asin)
          else:
              sender(asins)
              mongo_sender(asins, 'offer')
              asins.clear()


def get_mongo_sender(mongo_client, marketplace):

    def save_send_asns_to_mongo(asins_send, category):
        bulk_updates = []

        for asin in asins_send:
            item = {"_id": f"{asin}-{category}","time": datetime.now().replace(microsecond=0), "asin": asin, "type": category}

            bulk_updates.append(
                UpdateOne({"_id": item.pop("_id")}, {"$set": item}, upsert=True)
            )

        with ThreadPoolExecutor(max_workers=10) as executor:
            executor.submit(
                mongo_client.client[f"amazon_{marketplace.lower()}"]["asins_sended"].bulk_write, bulk_updates
            )
    return save_send_asns_to_mongo

def get_offer_sender(marketplace, broker_url, queue):
    if not ensure_target_country_match_queue(marketplace, queue):
        raise Exception(f"{marketplace} doest not math {queue}")

    connection = Connection(broker_url)

    def send_asins_to_offer_queue(chunk, condition='new'):
        spapi_update_item_offers.apply_async(
            args=(marketplace, chunk, condition),
            queue=queue,
            connection=connection)
        logger.debug(
            'Added spapi_update_item_offers(%s, %s, %s)',
            marketplace, chunk, condition
        )
    return send_asins_to_offer_queue

if __name__ == "__main__":
  analyze_products()
