# -*- coding: utf-8 -*-

import os
import sys
import datetime
import time
import json

import click
import redis
import dateparser

from em_celery import logger, init_db, get_product_service
from em_tasks.store.models import *


@click.command('Send coupang products crawl seed urls')
@click.option('-u', '--redis_uri', required=True, type=str, help='Redis URI.')
@click.option('-t', '--ttl', type=int, default=3, help='Product information alive days, default is 3.')
@click.option('-k', '--start_urls_key', required=True, type=str, default='coupang:start_urls', help='Start URLs key.')
def feed_seeds(redis_uri, start_urls_key='coupang:start_urls', ttl=300):
  init_db()
  feeder = CoupangProductsFetchSeedFeeder(redis_uri, start_urls_key, ttl)
  feeder.run()

class CoupangProductsFetchSeedFeeder():
  def __init__(self, redis_uri, start_urls_key, ttl=3):
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'www_coupang_com'
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 

  def load_source_product_urls(self):
    batch_size = 4000
    offset = 0
    sources = ["coupang"]


    while True:
        products = (
            ProductSource.select()
            .where(ProductSource.source.in_(sources))
            .limit(batch_size)
            .offset(offset)
        )
        for product in products:
            yield product.source_product_id, product.source_product_url

        if len(products) < batch_size:
            break
        offset = offset + batch_size

  def check_queue_full(self):
    max_length = 100000
    return self.r.llen(self.start_urls_key) > max_length

  def run(self):
    product_urls = {}
    # batch_size = 500
    batch_size = 5
    if self.check_queue_full():
      return None

    try:
      for product_id, url in self.load_source_product_urls():
          product_urls[product_id] = url
          if len(product_urls) < batch_size:
            continue

          self.process_products(product_urls)
          product_urls = {}
    except (KeyboardInterrupt, SystemExit):
      print('Exiting, please wait...')
    finally:
      if product_urls:
        self.process_products(product_urls)
        product_urls = []

  def process_products(self, product_urls):
    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, list(product_urls.keys()))
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if products:
        for pid, product in products.items():
          exist = product.get('existence', True)
          if not exist:
            product_urls.pop(pid, None)
            continue

          update_date_s = product.get('time')
          if not update_date_s:
            continue

          try:
            update_date = dateparser.parse(update_date_s).date()
            # Not expired, no need to update
            if update_date > self.expire_date:
              product_urls.pop(pid, None)
          except:
            continue

    urls = list(product_urls.values())
    for product_url in urls:
      breakpoint()

      print(self.r.llen(self.start_urls_key))
      self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
      queue_length = self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
      print(self.r.llen(self.start_urls_key))
      self.r.rpush(self.start_urls_key, json.dumps({'url': product_url}))
      print(self.r.llen(self.start_urls_key))
      logger.debug('Added %s', product_url)


if __name__ == "__main__":
  feed_seeds()
