# -*- coding: utf-8 -*-

import datetime
import json
import click
import redis
import dateparser

try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

from em_celery import logger, get_product_service, get_mongo

@click.command('Send ebay products crawl seed urls')
@click.option('-u', '--redis_uri', required=True, type=str, help='Redis URI.')
@click.option('-q', '--qps', type=float, default=20, help='Seeds add rate in seconds.')
@click.option('-t', '--ttl', type=int, default=3, help='Product information alive days, default is 3.')
@click.option('-k', '--start_urls_key', required=True, type=str, default='ebay_product:start_urls', help='Start URLs key.')
def feed_seeds(redis_uri, qps=20, start_urls_key='ebay_product:start_urls', ttl=3):
  feeder = EbayProductsFetchSeedFeeder(redis_uri, start_urls_key, qps, ttl)
  feeder.run()

class EbayProductsFetchSeedFeeder():

  def get_all_pids(self):
    pids = set()
    for url_info in self.r.lrange(self.start_urls_key, 0, -1):
        url_dict = json.loads(url_info)
        pids.add(self.extract_product_id(url_dict['url']))
    return pids

  def __init__(self, redis_uri, start_urls_key, qps, ttl=3):
    self.mongo_client = get_mongo()
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.qps = qps
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'ebay_us_products'
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl) 

  def run(self):
      pids = self.get_all_pids()
      for _, product in self.product_service.load_products("ebay_us_products"):
          product_url = product["url"]
          url = None
          if product_url.find('/p/') == -1:
            url = product_url
          if not url:
            continue

          pid = self.extract_product_id(product_url)
          if pid in pids:
              continue

          exist = product.get('existence', True)
          if not exist:
            continue

          update_date_s = product.get('date', product.get('time', None))
          if not update_date_s:
            continue

          try:
            update_date = dateparser.parse(update_date_s).date()
            # Not expired, no need to update
            if update_date > self.expire_date:
              continue
          except:
            continue

          self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
          logger.debug('Added %s', product_url)

  def extract_product_id(self, product_url):
    try:
      return urlparse(product_url).path.split('/').pop()
    except Exception as e:
      logger.exception(e)
if __name__ == "__main__":
  feed_seeds()
