# -*- coding: utf-8 -*-

import asyncio
import os
import datetime
import logging
import logging.handlers

from pymongo import UpdateOne
from concurrent.futures import ThreadPoolExecutor

import click
from kombu import Connection
from dropshipping.utils.utils import is_asin_valid
import dateutil
import dateutil.parser

from em_celery import logger, get_offer_service, get_mongo
from em_celery.tasks.spapi_update_item_offers_task import spapi_update_item_offers


from em_tasks.utils.offer_services import AmzOfferService
from em_tasks.utils.amz_products_analyzer_and_send import AmzProductsAnalyzerAndSend
from em_tasks.store.models import *

from em_celery import (
  init_db,
  get_config,
  get_amz_offer_filter_config,
  get_offer_service_config,
  get_product_service,
)


@click.command('Send spapi update item offers task to worker.')
@click.option('-b', '--broker_url', type=str, required=True, help='Celery worker broker URL.')
@click.option('-m', '--marketplace', type=str, default='us', help='Amazon marketplace to fetch offers.')
@click.option('-c', '--condition', type=str, default='new')
@click.option('-t', '--ttl', type=int, default=36, help='Offer alive hours, default is 36.')
@click.option('-f', '--force', is_flag=True, help='Force to update offers.')
def send_spapi_item_offers_update_task(broker_url, marketplace='us', condition='new', ttl=36, force=False):
  log_dir = os.path.join(os.path.expanduser('~'), '.em_celery', 'logs')
  if not os.path.isdir(log_dir):
    os.makedirs(log_dir)
  log_path = os.path.join(log_dir, 'spapi_update_item_offers_task_sender.log')
  level = logging.INFO
  max_bytes = 20 * 1024 ** 2
  fh = logging.handlers.RotatingFileHandler(
    log_path, maxBytes=max_bytes, backupCount=5)
  fh.setLevel(level)
  formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
  fh.setFormatter(formatter)
  logger.addHandler(fh)

  init_db()

  marketplace = marketplace.lower()
  filter_cond = get_amz_offer_filter_config(marketplace)
  offer_service_cfg = get_offer_service_config()
  offer_service = AmzOfferService(
    offer_service_cfg["host"],
    offer_service_cfg["port"],
    offer_service_cfg["user"],
    offer_service_cfg["password"],
    filter_cond,
  )

  product_service = get_product_service()

  sender = SpapiUpdateItemOffersTaskSender(broker_url, marketplace, condition, ttl, product_service, offer_service)
  sender.run()


class SpapiUpdateItemOffersTaskSender():
  def __init__(self, broker_url, marketplace, condition, ttl, product_service, offer_service):
    self.mongo_client = get_mongo()
    self.offer_service = get_offer_service()
    self.broker_url = broker_url
    self.marketplace = marketplace.lower()
    self.condition = condition
    self.ttl = ttl
    self.connection = Connection(broker_url)
    self.queue = 'SpapiItemOffersUpdate_{}'.format(marketplace.upper())
    self.offer_type = 'lowest_offer_listings'
    self.last_send_time = None
    self.products_filter = AmzProductsAnalyzerAndSend(marketplace, product_service, offer_service, self.mongo_client)

  def run(self):
    asins_buf = []
    batch_size = 500 

    for gen in self.products_filter.run():
        for asin in gen:
            if not is_asin_valid(asin):
                continue

            asins_buf.append(asin)
            if len(asins_buf) < batch_size:
                continue

            asins_send = self.process_products(asins_buf)
            if asins_send is not None:
                self.save_send_asns_to_mongo(asins_send, 'offer')

            asins_buf = []

        if asins_buf:
            asins_send = self.process_products(asins_buf)
            if asins_send is not None:
                self.save_send_asns_to_mongo(asins_send, 'offer')
            asins_buf = []

  def save_send_asns_to_mongo(self, asins_send, category):
    bulk_updates = []

    for asin in asins_send:
        item = {"_id": f"{asin}-{category}", "time": datetime.datetime.now().replace(microsecond=0), "asin": asin, "type": category}

        bulk_updates.append(
            UpdateOne({"_id": item.pop("_id")}, {"$set": item}, upsert=True)
        )

    with ThreadPoolExecutor(max_workers=10) as executor:
         executor.submit(
            self.mongo_client.client[f"amazon_{self.marketplace.lower()}"]["asins_sended"].bulk_write, bulk_updates
        )

  def process_products(self, asins):
    if True:
      now = datetime.datetime.utcnow()
      offer_expire_time = now - datetime.timedelta(hours=self.ttl)

      asins_without_offer = dict()
      offers = dict()
      result = self.offer_service.search_offers(
        self.offer_type, asins, self.marketplace, self.condition)
      if not result:
        asins_without_offer = list(asins)
      else:
        while isinstance(result, dict) and 'hits' in result:
          result = result['hits']
        if isinstance(result, list):
          for offer in result:
            if not offer:
              continue

            offers[offer['_source']['asin']] = offer['_source']
        else:
          offers = result
        for asin in asins:
          if asin not in offers or not offers[asin]:
            asins_without_offer[asin] = None
            continue

          offer = offers[asin]
          offer_time_s = offer.get('time', None)
          if not offer_time_s:
            asins_without_offer[asin] = None
            continue

          try:
            offer_time = dateutil.parser.parse(offer_time_s)

            # Offer expired
            if offer_time < offer_expire_time:
              asins_without_offer[asin] = None
              continue
          except Exception as e:
            asins_without_offer[asin] = None

          logger.debug('[ASINHasOffer] %s', asin)

        asins_without_offer = list(asins_without_offer.keys())

    chunks = [asins_without_offer[x:x + 20] for x in range(0, len(asins_without_offer), 20)]
    for chunk in chunks:
      spapi_update_item_offers.apply_async(
        args=(self.marketplace, chunk, self.condition),
        queue=self.queue,
        connection=self.connection)
      logger.debug(
        'Added spapi_update_item_offers(%s, %s, %s)',
        self.marketplace, chunk, self.condition
      )

    return asins_without_offer


if __name__ == "__main__":
  send_spapi_item_offers_update_task()
